### Cyclic Redundancy Codes (CRCs)

Putem reprezenta datele transmise ca pe un număr, care trebuie
împărțit la o valoare fixă, prestabilită.
Restul împărțirii este valoarea pe care o putem introduce în header,
iar receptorul poate verifica dacă datele primite, împărțite la
valoarea convenită, dau același rest.

```
De exemplu, atât transmițătorul cât și receptorul sunt de acord să folosească
13131 ca și împărțitor.
Transmițătorul dorește să trimită datele "123131"

REST = 123131 % 13131 = 200

|DELIM|PAYLOAD|REST|DELIM|

```

În practică, nu folosim numere, ci polinoame, printre altele fiind mult mai ușor de lucrat cu ele (nu o să avem carry). **Cyclic Redundancy Codes (CRC) reprezintă restul
împărțirii polinomiale modulo 2 a datelor pe care vrem să le trimitem**. 
Putem vedea payload-ul ca și reprezentarea unui polinom.
```
PAYLOAD=    'H'      'i'     '!'
          01001000 01101001 00100001
```

Cu reprezentarea matematică

\\( x^{22} + x^{19} + x^{14} + x^{13} + x^{11} +x^{8} +x^{5} + x^{0} \\)

De ce modulo 2 (inelul claselor de resturi modulo 2)? Deoarce vrem ca
indicii în urma calculelor să fie 1 sau 0, altfel, atunci când facem împărțirea,
am ajunge la valori reale, iar noi putem folosi doar valori binare.

Operațiile în
acest inel sunt echivalente cu XOR (e.g. 1 + 1 = 0).  În funcție de polinomul la
care o să ne raportăm, avem diferite implementări de CRC.
[Aici](https://en.wikipedia.org/wiki/Cyclic_redundancy_check#Polynomial_representations_of_cyclic_redundancy_checks)
gășiți mai multe exemple de polinoame si la ce sunt folosite. CRC 32 folosește
următorul polinom:

\\( x^{32} + x^{26} + x^{23} + x^{22} + x^{16} + x^{12} + x^{11} + x^{10} + x^{8} + x^{7} + x^{5} + x^{4} + x^{2} + x + 1 \\)

cu reprezentarea in binar si hexa:
```
0xEDB88320 	11101101-10111000_10000011_00100000
            x^0
```
Cum avem doar 32 de biți, nu ne interesează indicele lui x^32. Polinomul a fost ales astfel încât să funcționeze bine în cazul erorilor în rafală.

Pentru string-ul `"123456789"`, valoarea CRC32 este `0xCBF43926`.

Un exemplu de împărțire de polinoame modulo 2 este acesta:

```
10011 ) 11010110110000 = Bits of payload
=Poly   10011,,.,,....
        -----,,.,,....
         10011,.,,....  (operația de xor cand primul bit e 1)
         10011,.,,....
         -----,.,,....
          00001.,,....  (când primul bit e zero, doar face un shift stânga
          00000.,,....   pentru a lua următorul indice de exponent)
          -----.,,....
           00010,,....
           00000,,....
           -----,,....
            00101,....
            00000,....
            -----,....
             01011....
             00000....
             -----....
              10110...
              10011...
              -----...
               01010..
               00000..
               -----..
                10100.
                10011.
                -----.
                 01110
                 00000
                 -----
                  1110 = Remainder = The CRC!
```

O posibilă implementare a algoritmului CRC32 este următoarea:

```C
uint32_t compute_crc32(const char *buffer, size_t len)
{
    /* unsigned char *buffer contine payload-ul, len este lungimea acestuia */
    /* Prin conventie crc-ul initial are toti bitii setati pe 1 */
    uint32_t crc = ~0; // 0xffffffff
    const uint32_t POLY = 0xEDB88320;

    /* Parcurgem fiecare byte din buffer */
    while(len--)
    {
        /* crc contine restul impartirii la fiecare etapa */
        /* nu ne intereseaza catul */
        /* adunam urmatorii 8 bytes din buffer */
        crc = crc ^ *buffer++;
        for( int bit = 0; bit < 8; bit++ )
        {
            /* 10011 ) 11010110110000 = Bytes of payload
                =Poly   10011,,.,,....
                        -----,,.,,....
                         10011,.,,....  (operatia de xor cand primul bit e 1)
                         10011,.,,....
                         -----,.,,....
                          00001.,,....  (asta e noua valoare a lui crc) (crc >> 1) ^ POLY
            */
            if( crc & 1 )
                crc = (crc >> 1) ^ POLY;
            else 
                /* 10011 ) 11010110110000 = Bytes of payload
                    =Poly   10011,,.,,....
                            -----,,.,,....
                             10011,.,,....  
                             10011,.,,....
                             -----,.,,....
                              00001.,,....  primul bit e 0, 
                              00000.,,....  
                              -----.,,....
                               00010,,.... am facut shift la dreapta, pentru ca suntem pe **little endian**
                */
                crc = (crc >> 1);
        }
    }
    // Prin conventie, o sa facem flip la toti bitii
    crc = ~crc;
    return crc;
}
```

---
**Note**

Pentru o înțelegere mai bună a matematicii din spatele CRC, vă recomandăm [următorul video (40 min)](https://www.youtube.com/watch?v=izG7qT0EpBw).

---
